-- Conditions as tables

CONDITION_HANDLERS = false

--[[Quick reference
- Condition tables consist of groups
- If ONE of provided groups is true, then condition is true
- Groups consist of multiple check calls
- Each check call is a list of parameters for check function
- All parameters in check call are separately checked against target; all must succeed
- In group, all check calls must succeed for group to succeed
- To create inverted (=~ or !=) check call, prefix it with "Not"

So, it looks something like this:
{	
	{
		SomeCheck = {"SomeValue", "SomeValue2"}, -- SomeCheck against target succeeds for "SomeValue" and "SomeValue2"
		NotAnotherCheck = {"Something", "NotTrue"} -- AnotherCheck fails BOTH "Something" and "NotTrue"
	},

	{
		-- Group 2, with same format
	},

	-- ... and so on
}

Or, new format:
{
	{
		{"SomeCheck", "SomeValue"},
		{"SomeCheck", "SomeValue2"},
		{"NotAnotherCheck", "Something"},
		{"NotAnotherCheck", "NotTrue"}
	},

	{
		-- Group 2, with same format
	},

	-- ... and so on
}
]]

-- Checks if conditions match for target
function CheckConditions(conditions, target, callback)
	if conditions == nil then return true end
	--DebugMessage("Not nil conditions")

	if type(conditions) == "function" then
		return conditions(target)
	end

	local postpone = false

	local first = conditions[1]
	if type(first) == "string" then -- We only have one condition, which is in "compact format"
		local fail = false

		if first:find("Not") == 1 then
			inverse = true
			first = first:sub(4)
		end

		local func = AllConditions[first]
		if func ~= nil then
			local inverse = false

			local param = conditions[2]
			local result, p1 = func(param, target, callback) -- If more than one parameters are used, this need nested table :(

			if result == false and inverse == false then fail = true
			elseif result == true and inverse == true then fail = true end
			if p1 then postpone = true end
		else
			DebugMessage("Warning, no condition function found!")
			LuaDebugCallStack()
			fail = true
		end

		if not fail and not postpone then return true end
		if postpone then return false, true end
		return false
	end

	for id,group in ipairs(conditions) do
		local fail = false

		if group[1] ~= nil then
			for i,cond in pairs(group) do
				local name = cond[1]

				local inverse = false
				if name:find("Not") == 1 then
					inverse = true
					name = name:sub(4)
				end

				local func = AllConditions[name]
				if func ~= nil then
					local param = cond[2]
					local result, p1 = func(param, target, callback) -- Ff more than one parameters are used, this need nested table :(

					if result == false and inverse == false then fail = true
					elseif result == true and inverse == true then fail = true end
					if p1 then postpone = true end
				else
					DebugMessage("Warning, no condition function found!")
					LuaDebugCallStack()
					fail = true
				end
			end
		else
			for name,params in pairs(group) do
				local inverse = false
				if name:find("Not") == 1 then
					inverse = true
					name = name:sub(4)
				end

				--DebugMessage("Function name is "..name)
				local func = AllConditions[name]
				if func ~= nil then
					for _,param in pairs(params) do
						local result, p1 = func(param, target, callback)
						if result == false and inverse == false then fail = true
						elseif result == true and inverse == true then fail = true end
						if p1 then postpone = true end
					end
				else
					DebugMessage("Warning, no condition function found!")
					LuaDebugCallStack()
					fail = true
				end

				--DebugMessage("inverse is "..tostring(inverse))
				--DebugMessage("fail is "..tostring(fail))
			end
		end

		if not fail and not postpone then return true end
	end

	if postpone then return false, true end

	return false
end
--DebugMessage("Load condition system!")

-- Callback system - use if needed, currently ObjVars are used when possible
-- NOTE 23.6.2016: Figure out if this is used anywhere -bensku
WaitingCallbacks = {}
SympathyData = {}
TraitData = {}
ConcernData = {}

function RegisterConditionEvents()
	Sympathyrecievers.Conditions = function (callback, sender, target, sympathy)
		SympathyData[callback][name] = sympathy
		CheckCallback(callback)
	end

	TraitRecievers.Conditions = function (callback, sender, name, level)
		TraitData[callback][name] = level
		CheckCallback(callback)
	end

	ConcernRecievers.Conditions = function (callback, sender, name, level)
		ConcernData[callback][name] = level
		CheckCallback(callback)
	end

	CONDITION_HANDLERS = true
end

function WaitCallback(callback, func)
	WaitingCallbacks[callback] = func
	CheckCallback(callback)
end

function CheckCallback(callback)
	local func = WaitingCallbacks[callback]
	local sympathy = SympathyData[callback]
	local trait = TraitData[callback]
	local concern = ConcernData[callback]

	local dirty = false
	if sympathy ~= nil then
		for k,v in pairs(sympathy) do
			if v == "dirty" then dirty = true end
		end
	end
	if trait ~= nil then
		for k,v in pairs(trait) do
			if v == "dirty" then dirty = true end
		end
	end
	if concern ~= nil then
		for k,v in pairs(concern) do
			if v == "dirty" then dirty = true end
		end
	end

	if not dirty then
		func()
		WaitingCallbacks[callback] = nil
	end
end

AllConditions = {

	Equals = function (param, target)
		return param == target
	end,

	TemplateEquals = function (param, target)
		if GetUserdataType(target) == "GameObj" then
			return param:GetCreationTemplateId() == target:GetCreationTemplateId()
		end
		return false
	end,

	HaveMet = function (param, target)
		if GetUserdataType(target) == "GameObj" then
			local npcId
			if GetUserdataType(param) == "GameObj" then
				npcId = param:GetCreationTemplateId()
			elseif type(param) == "string" then
				npcId = param
			else
				return false
			end
			--DebugMessage("npcId is "..npcId)

			local meetTable = target:GetObjVar("NPCMeetTable")
			if meetTable ~= nil then
				return meetTable[param]
			else
				target:SetObjVar("NPCMeetTable", meetTable)
			end
		end

		return false
	end,

	SkillLevel = function (param, target)
		if GetUserdataType(target) == "GameObj" and type(param) == "table" then
			local level = target:GetSkillLevel(param.skill or param[1])
			local simpleLvl = target.level or target[2]
			if simpleLvl ~= nil then return level == simpleLvl end

			local min = param.minLevel or 0
			local max = param.maxLevel or 5000
			--DebugMessage("min: "..tostring(min)..", max: "..tostring(max))
			--DebugMessage("level is "..level)
			--DebugMessage("Result is "..tostring((level >= min) and (level <= max)))
			return level >= min and level <= max
		end

		return false
	end,

	ObjVar = function (param, target)
		if GetUserdataType(target) == "GameObj" and type(param) == "string" then
			return target:HasObjVar(param)
		end
		return false
	end,

	
	-- result = CheckConditions({"ObjVarValue",{ObjVar="Var1",Value=123456}}, target)
	ObjVarValue = function (param, target)
		return target:GetObjVar(param.ObjVar)==param.Value
	end,

	Trait = function (param, target)
		if type(param) ~= "string" then return false end
		if GetUserdataType(target) == "GameObj" then
			--[[if TraitData[callback] ~= nil and TraitData[callback][param] ~= nil then
				target:SendMessage("GetTrait", callback, this, param)
				TraitData[callback][param] = "dirty" -- Dirty flag
				return false, true
			else
				return TraitData[callback][param] >= param
			end]]

			if target:HasObjVar("RelTraits") then
				local traits = target:GetObjVar("RelTraits")
				--DebugMessage("Trait is "..tostring(traits[param] ~= nil))
				return traits[param] ~= nil
			end
			return false
		elseif type(target) == "table" then
			return target[param] ~= nil
		end
	end,

	TraitLevel = function (param, target, callback)
		if type(param) ~= "string" then return false end
		if GetUserdataType(target) == "GameObj" then
			if target:HasObjVar("RelTraits") then
				local traits = target:GetObjVar("RelTraits")
				local level = traits[param.Name or param[1]]

				local simpleLvl = target.Level or target[2]
				if simpleLvl ~= nil then return level == simpleLvl end

				local min = param.MinLevel or 0
				local max = param.MaxLevel or 1000
				return level >= min and level <= max
			end
			return false
		elseif type(target) == "table" then
			local level = target[param.Name]
			if target.Level ~= nil then return level == target.Level end

			local min = param.MinLevel or 0
			local max = param.MaxLevel or 1000
			return level >= min and level <= max
		end
	end,

	Concern = function (param, target)
		if type(param) ~= "string" then return false end
		if GetUserdataType(target) == "GameObj" then
			if target:HasObjVar("RelConcerns") then
				local concerns = target:GetObjVar("RelConcerns")
				return concerns[param]
			end
			return false
		elseif type(target) == "table" then
			return target[param] ~= nil
		end
	end,

	ConcernLevel = function (param, target, callback)
		if type(param) ~= "string" then return false end
		if GetUserdataType(target) == "GameObj" then
			if target:HasObjVar("RelConcerns") then
				local concerns = target:GetObjVar("RelConcerns")
				local level = concerns[param.Name]

				if target.Level ~= nil then return level == target.Level end

				local min = param.MinLevel or 0
				local max = param.MaxLevel or 1000
				return level >= min and level <= max
			end
			return false
		elseif type(target) == "table" then
			local level = target[param.Name]
			if target.Level ~= nil then return level == target.Level end

			local min = param.MinLevel or 0
			local max = param.MaxLevel or 1000
			return level >= min and level <= max
		end
	end,

	HasQuest = function (param, target)
		local quests = target:GetObjVar("QuestTable")
		if quests == nil then return false end

		for i,e in pairs(quests) do
			if e.QuestName == param then
				return true
			end
		end

		return false
	end,

	QuestState = function (param, target)
		local quests = target:GetObjVar("QuestTable")
		if quests == nil then return false end

		for i,e in pairs(quests) do
			if e.QuestName == param[1] then
				return e.CurrentTask == param[2]
			end
		end

		return false
	end,

	HasFolio = function (param, target)
		local entries = target:GetObjVar("ArcanimaFolio")
		return entries[param] ~= nil
	end
}