--[[
File: globals_incl_colorfuncs
Helper Functions to deal with colors
Needed by the Dye Merchant and the Color Editor
~Yorlik
--]]

-- convert a hex string like FF0000 (red) into a table of RGB numbers like {r=255, g=0, b=0} (red)
function Hex2Rgb(hex)
	local red 	= tonumber(string.sub(hex,1,2),16)
    local green = tonumber(string.sub(hex,3,4),16)
    local blue 	= tonumber(string.sub(hex,5,6),16)
    return {r=red, g=green,b=blue}
end
-- inverse of the above
function Rgb2Hex(rgb)
	return string.format("%02X%02X%02X",rgb.r,rgb.g,rgb.b)
end

-- convert a hex string like FF0000 (red) into a table of RGB percentages like {r=100, g=0, b=0} (red)
function Hex2RgbPct(hex)
	local red 	= tonumber(string.sub(hex,1,2),16)
    local green = tonumber(string.sub(hex,3,4),16)
    local blue 	= tonumber(string.sub(hex,5,6),16)
    return {r=red/255, g=green/255,b=blue/255}
end

-- weighed color mix. Returns a Hex String with Color Mix result.
function MixHues(hue1,hue2,weight1, weight2)
	if weight1==nil then weight1=1 end
	if weight2==nil then weight2=1 end

    local hue1red = tonumber(string.sub(hue1,1,2),16)*weight1
    local hue1green = tonumber(string.sub(hue1,3,4),16)*weight1
    local hue1blue = tonumber(string.sub(hue1,5,6),16)*weight1
    
    local hue2red = tonumber(string.sub(hue2,1,2),16)*weight2
    local hue2green = tonumber(string.sub(hue2,3,4),16)*weight2
    local hue2blue = tonumber(string.sub(hue2,5,6),16)*weight2
   	local divisor = weight1 + weight2
    local newred =  math.floor( (hue1red   + hue2red) 	/ divisor)
    local newgreen =math.floor( (hue1green + hue2green) / divisor)
    local newblue = math.floor( (hue1blue  + hue2blue) 	/ divisor)       
   
    return string.format("%02X%02X%02X",newred,newgreen,newblue)
end

-- extract HSL (Hue, Saturation, Luminosity) from color hex codes
-- HSL values between 0 and 1
function Hex2Hsl(hex)
	local rgb=Hex2RgbPct(hex)
	local Cmax = math.max(rgb.r, rgb.g, rgb.b)
	local Cmin = math.min(rgb.r, rgb.g, rgb.b)
	local Delta=Cmax-Cmin

	-- Luminosity
	local Lum = (Cmin+Cmax)/2

	-- Saturation
	local Sat=0
	if Delta==0 then 
			Sat=0 
		else
			Sat=Delta/(1-math.abs(2*Lum-1))
	end

	-- Hue
	local Hue=0
	if Delta==0 then 
			Hue=0 
		elseif Cmax==rgb.r then
			Hue = ((rgb.g - rgb.b) / Delta )%6
		elseif Cmax==rgb.g then
			Hue = ( (rgb.b - rgb.r) / Delta   ) + 2
		elseif Cmax==rgb.b then
			Hue = ( (rgb.r - rgb.g) / Delta   ) + 4
	end
	Hue=Hue*60
	
	return {h=Hue,s=Sat,l=Lum}
end

-- inverse of above function. 
-- Returns Hex String From HSL numbers
function Hsl2Hex(hsl)
	local H,S,L=hsl.h%360 or 0 ,hsl.s or 0,hsl.l or 0
	local C = (1-math.abs(2*L-1))*S
	local X = C*(1-math.abs((H/60)%2-1))
	local m = L-C/2
	local rgb={}

	--DebugMessage("Hsl2Hex: H:"..H..", S:"..S..", L:"..L..", C:"..C..", X:"..X..", m:"..m)

	if 0<=H and H<60 then
			rgb={C,X,0}
		elseif 60<=H and H<120 then
			rgb={X,C,0}
		elseif 120<=H and H<180 then
			rgb={0,C,X}
		elseif 180<=H and H<240 then
			rgb={0,X,C}
		elseif 240<=H and H<300 then
			rgb={X,0,C}
		--elseif 300<=H and H<360 then
		elseif 300<=H then
			rgb={C,0,X}
	end
	--DebugMessage("rgb[1]:"..tostring(rgb[1])..", rgb[2]:"..tostring(rgb[2])..", rgb[3]:"..tostring(rgb[3]))
	rgb[1]=rgb[1]+m
	rgb[2]=rgb[2]+m
	rgb[3]=rgb[3]+m

	return string.format("%02X%02X%02X",math.floor(rgb[1]*255+0.5),math.floor(rgb[2]*255+0.5),math.floor(rgb[3]*255+0.5))
end