--[[
file: object_move.lua
when attached to an object and the object receives the message below 
it moves the object from its position and rotation 
to a target location and rotation
at a given movement speed

-- message example:
-- "o" is our game object... 
o:SendMessage(
	"MoveObject",
	{	-- actions at begin and end of animation
		first_action	 =	nil, -- name of a behavior that gets attached to the object before the first frame
		final_action	 =	nil, -- name of a behavior that gets attached to the object after the last frame
		-- framerate and movement speed
		ms_per_frame	 = 40, 	-- milliseconds per frame. Defines framerate. 40 ms = 25 FPS
		m_per_sec 		 = 1, 	-- meters (=units) per second speed along the trajectory
								-- Since we lift the object 10 units at 1m/sec speed, 
								-- the animation will last 10 seconds.
		-- Start and end locations
		startLoc		 = o:GetLoc(), -- current object position
		endLoc  		 = Loc(
								o:GetLoc().X,
								o:GetLoc().Y+10,
								o:GetLoc().Z
								),	-- object destination 10 meters above original position
		-- begin and end rotation
		startRot	 	 = o:getRotation(),	-- Rotation of the object
		endRot			 = Loc(
								o:getRotation().X,
								o:getRotation().Y+180,
								o:getRotation().Z,
								), -- we will turn the object by 180 degree along the path
	})
--]]


-- Moves object CONSTANTLY AT A LINEAR TRAJECTORY in a controlled manner 
-- at a given framerate, speed and rotation
-- if the server lags, frames get automatically dropped to keep up speed
-- Locations and Rotation values are absolute
RegisterEventHandler(EventType.Message,"MoveObject",
	function(args)
		DebugMessage("Moving Object --------")

		--- Setup and basic calculations
		local startLoc = args.startLoc or this:GetLoc() -- my current location
		local dist = startLoc:Distance(args.endLoc) -- distance to destination
		local durationTicks = (dist/args.m_per_sec)* 10000000 -- ten million
		local frames = (durationTicks/args.ms_per_frame)/10000000 -- How many frames ?
		DebugMessageF("dist %s, time_to_travel %s, frames %s",dist,time_to_travel,frames)

		local f =  math.max(math.floor(frames),1)
		DebugMessageF("Number of Frames = %d",f)

		-- preparation function, initializer, initial effects or whatever you desire
		if args.before_action ~= nil then 
			this:AddModule(args.first_action) --- first function, setup, sounds, etc
		end
		-- prepare animation parameters
		local startTicks = DateTime.UtcNow.Ticks
		local endTicks 	 = startTicks+durationTicks
		local params = {
				ms_per_frame = args.ms_per_frame 	,
				startTicks	 = startTicks 			, 
				endTicks	 = endTicks 			,
				durationTicks=durationTicks 		, 
				-- Locations are absolute
				startLoc	 = args.startLoc 	or this:GetLoc()		, 
				endLoc		 = args.endLoc 		or this:GetLoc()		,
				-- rotation is relative to original Location
				startRot	 = args.startRot 	or this:GetRotation()	,
				endRot		 = args.endRot 		or this:GetRotation()	,
				final_action = args.final_action	,
			}
		-- Start Animation
		this:FireTimer("Animation",params)
	end)

-- called each frame until end
function updateposition(args)
			--DumpTable(args,1)
			--DebugMessage("UpDate Pos --------")
			local nowTicks = DateTime.UtcNow.Ticks
			local percent = (args.endTicks-nowTicks)/args.durationTicks
			local newLoc = args.endLoc:LerpTo(args.startLoc,percent)
			local newRot = args.endRot:LerpTo(args.startRot,percent)
			-- new position and rotation
			this:SetWorldPosition(newLoc)
			this:SetRotation(newRot)

			--DebugMessageF("Ticks: Now: %s, End: %s, Loc: %s",nowTicks,args.endTicks,args.endLoc)

			-- if the new time is newer than the calculated last frame ... end of game
			if nowTicks>=args.endTicks then --- end of animation
				-- final Rotation and Position
				this:SetWorldPosition(args.endLoc)
				this:SetRotation(args.endRot)

				DebugMessage("Last Frame reached %s",args.endLoc)
				-- finishing action if any
				if args.final_action ~= nil then
					DebugMessage("Final Action ..." )
					this:AddModule(args.final_action) --- end function, processing damage etc
				end
				-- End of the show
				this:RemoveTimer("Animation")
				return -- no more frames
			end

			-- scheduling next frame
			this:ScheduleTimerDelay(TimeSpan.FromMilliseconds(ms_per_frame),"Animation",args) -- passing args unchanged

		end
-- Animation Timer 		
RegisterEventHandler(EventType.Timer,"Animation",updateposition)

	


